@@ -19,70 +19,64 @@
 #'
 #' @param g An \code{igraph} graph object
 #' @param type Character string; either \code{local}, \code{nodal}, or
-#'   \code{global}. Default: \code{local}
+#'   \code{global} (default: \code{local})
 #' @param weights Numeric vector of edge weights; if \code{NULL} (the default),
 #'   and if the graph has edge attribute \code{weight}, then that will be used.
 #'   To avoid using weights, this should be \code{NA}.
-#' @param use.parallel Logical indicating whether or not to use \code{foreach}.
-#'   Default: \code{TRUE}
-#' @param A Numeric matrix; the adjacency matrix of the input graph. Default:
-#'   \code{NULL}
-#' @param D Numeric matrix; the graph's \dQuote{distance matrix}
+#' @param use.parallel Logical indicating whether or not to use \code{foreach}
+#'   (default: \code{TRUE})
+#' @param A Numeric matrix; the (weighted or unweighted) adjacency matrix of the
+#'   input graph (default: \code{NULL})
 #' @export
 #' @importFrom Matrix rowSums
-#' @importFrom foreach getDoParRegistered
-#' @importFrom doParallel registerDoParallel
 #'
 #' @return A numeric vector of the efficiencies for each vertex of the graph
 #'   (if \emph{type} is \code{local|nodal}) or a single number (if \emph{type}
 #'   is \code{global}).
 #'
 #' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
-#' @references Latora, V. and Marchiori, M. (2001) Efficient behavior of
-#'   small-world networks. \emph{Phys Rev Lett}, \bold{87.19}, 198701.
-#'   \url{https://dx.doi.org/10.1103/PhysRevLett.87.198701}
-#' @references Latora, V. and Marchiori, M. (2003) Economic small-world
-#'   behavior in weighted networks. \emph{Eur Phys J B}, \bold{32}, 249--263.
-#'   \url{https://dx.doi.org/10.1140/epjb/e2003-00095-5}
+#' @references Latora V., Marchiori M. (2001) \emph{Efficient behavior of
+#'   small-world networks}. Phys Rev Lett, 87.19:198701.
+#' @references Latora V., Marchiori M. (2003) \emph{Economic small-world
+#'   behavior in weighted networks}. Eur Phys J B, 32:249-263.
 
 efficiency <- function(g, type=c('local', 'nodal', 'global'), weights=NULL,
-                       use.parallel=TRUE, A=NULL, D=NULL) {
+                       use.parallel=TRUE, A=NULL) {
   stopifnot(is_igraph(g))
   i <- NULL
   weights <- check_weights(g, weights)
 
   type <- match.arg(type)
   if (type == 'local') {
-    e.attr <- weighted <- NULL
     if (is.null(weights)) {
-      e.attr <- 'weight'
+      if (is.null(A)) A <- as_adj(g, names=FALSE, attr='weight')
       weighted <- TRUE
+    } else {
+      A <- as_adj(g, names=FALSE, sparse=FALSE)
+      weighted <- NULL
     }
-    if (is.null(A)) A <- as_adj(g, names=FALSE, sparse=FALSE, attr=e.attr)
-    eff <- rep(0, dim(A)[1L])
-    verts <- which(rowSums((A > 0) + 0) > 1)
+    eff <- rep(0, nrow(A))
+    nodes <- which(rowSums((A > 0) + 0) > 1)
     X <- apply(A, 1, function(x) which(x > 0))
-    if (is.matrix(X)) X <- as.list(data.frame(X))   # If the graph is complete
 
-    if (length(verts) > 0) {
-      `%d%` <- `%do%`
+    if (length(nodes) > 0) {
       if (isTRUE(use.parallel)) {
-        if (!getDoParRegistered()) {
-          cl <- makeCluster(getOption('bg.ncpus'))
-          registerDoParallel(cl)
+        eff[nodes] <- foreach (i=nodes, .combine='c') %dopar% {
+          g.sub <- graph_from_adjacency_matrix(A[X[[i]], X[[i]]], mode='undirected', weighted=weighted)
+          efficiency(g.sub, 'global', weights=weights)
+        }
+      } else {
+        for (i in nodes) {
+          g.sub <- graph_from_adjacency_matrix(A[X[[i]], X[[i]]], mode='undirected', weighted=weighted)
+          eff[i] <- efficiency(g.sub, 'global', weights=weights)
         }
-        `%d%` <- `%dopar%`
-      }
-      eff[verts] <- foreach(i=verts, .combine='c') %d% {
-        g.sub <- graph_from_adjacency_matrix(A[X[[i]], X[[i]]], mode='undirected', weighted=weighted)
-        efficiency(g.sub, 'global', weights=weights)
       }
     }
   } else {
-    if (is.null(D)) D <- distances(g, weights=weights)
-    Nv <- dim(D)[1L]
+    D <- distances(g, weights=weights)
+    Nv <- nrow(D)
     Dinv <- 1 / D
-    eff <- colSums(Dinv * is.finite(Dinv), na.rm=TRUE) / (Nv - 1)
+    eff <- colSums(Dinv * is.finite(Dinv), na.rm=T) / (Nv - 1)
     if (type == 'global') eff <- sum(eff) / length(eff)
   }
   return(eff)
